/*  naumdesign.h : header file for multidesign version of nauty()  */
/*  Version 1.0, B.D. McKay Feb 2023 */

#include "nauty.h"      /* which includes stdio.h */

typedef setword design;
/* The internal representation used by nauty is an array of setwords,
   the first containing the number of multiblocks and the others being
   pairs (block,mult), where block is a subset of BLOCKMASK giving
   the block and mult is a subset of MULTMASK giving its multiplicity.
   The blocks are distinct and in sorted order. Multiplicities are > 0.
   The number of points is passed to procedures in the parameter n. */

#ifndef MULTBITS
#define MULTBITS 8  /* Number of low-order bits used for multiplicity */
#endif
#define MULTMASK (((design)1<<MULTBITS)-1)  /* One less than a power of 2 */
#define MAXMULT MULTMASK
#define BLOCKMASK (~((design)0) ^ MULTMASK)  /* Other bits */
#define MAXPOINTS (WORDSIZE-MULTBITS)
#ifndef MAXBLOCKS
#define MAXBLOCKS (4*(1<<MULTBITS))
#endif

#define BLOCK(x) ((x) & BLOCKMASK)
#define MULT(x)  ((x) & MULTMASK)
#define BLOCKLEN(x) POPCOUNT(BLOCK(x))

#ifdef __cplusplus
extern "C" {
#endif

extern void shell_des(design*,int);
extern boolean cheapautom_des(int*,int,boolean,int);
extern boolean isautom_des(graph*,int*,boolean,int,int);
extern void refine_des (graph*,int*,int*,int,int*,int*,set*,int*,int,int);
extern void refine1_des (graph*,int*,int*,int,int*,int*,set*,int*,int,int);
extern int testcanlab_des(graph*,graph*,int*,int*,int,int);
extern void updatecan_des(graph*,graph*,int*,int,int,int);
extern int targetcell_des(graph*,int*,int*,int,int,boolean,int,int,int);
extern void naumdesign_check(int,int,int,int);
extern void naumdesign_freedyn(void);
extern void naumdesign(design*,int*,int*,set*,
                                int*,optionblk*,statsblk*,int,design*);
extern void blockautom(design*,int*,int*,int);

extern dispatchvec dispatch_mdesign;

#ifdef __cplusplus
}
#endif

#define DEFAULTOPTIONS_MDES(options) optionblk options \
    = {0,TRUE,FALSE,FALSE,TRUE,FALSE,CONSOLWIDTH,NULL,NULL,NULL, \
	NULL,NULL,NULL,NULL,0,0,1,0,&dispatch_mdesign,FALSE,NULL}

#define DLINELEN(n,b) (3+((b)*1L*(n)+5)/6)   /* Length of d-format line */

#ifndef MAXB
#define MAXB 256  /* Only used in dio.c */
#endif
#undef MAXL
#define MAXL DLINELEN(MAXN,MAXB)    
#if MAXL < 50000
#undef MAXL
#define MAXL 50000
#endif
